@@ -27,6 +27,8 @@
 static GSList *fields = NULL;
 static guint n_fields;
 
+static gboolean disable_changed = TRUE;
+
 /* expand %N in command to fields values */
 static GString *
 expand_action (gchar * cmd)
@@ -71,6 +73,7 @@ expand_action (gchar * cmd)
                 case YAD_FIELD_MFILE:
                 case YAD_FIELD_MDIR:
                 case YAD_FIELD_DATE:
+                case YAD_FIELD_ICON:
                   buf = escape_char ((gchar *) gtk_entry_get_text (GTK_ENTRY (g_slist_nth_data (fields, num))), '"');
                   arg = g_shell_quote (buf ? buf : "");
                   g_free (buf);
@@ -195,6 +198,7 @@ set_field_value (guint num, gchar *value)
     case YAD_FIELD_FILE_SAVE:
     case YAD_FIELD_DIR_CREATE:
     case YAD_FIELD_DATE:
+    case YAD_FIELD_ICON:
       gtk_entry_set_text (GTK_ENTRY (w), value);
       break;
 
@@ -369,6 +373,33 @@ set_field_value (guint num, gchar *value)
     }
 }
 
+static void
+parse_cmd_output (gchar *data)
+{
+  guint i = 0;
+  gchar **lines = g_strsplit (data, "\n", 0);
+
+  disable_changed = TRUE;
+  while (lines[i] && lines[i][0])
+    {
+      gint fn;
+      gchar *ptr = lines[i];
+
+      while (isblank (*ptr)) ptr++;
+
+      if (isdigit (*ptr))
+        {
+          gchar **ln = g_strsplit (ptr, ":", 2);
+          fn = g_ascii_strtoll (ln[0], NULL, 10);
+          if (fn && ln[1])
+            set_field_value (fn - 1, ln[1]);
+          g_strfreev (ln);
+        }
+      i++;
+    }
+  disable_changed = FALSE;
+}
+
 static void
 button_clicked_cb (GtkButton * b, gpointer d)
 {
@@ -376,43 +407,23 @@ button_clicked_cb (GtkButton * b, gpointer d)
 
   if (action && action[0])
     {
+      GString *cmd;
       if (action[0] == '@')
         {
-          gchar *data;
+          gchar *data = NULL;
           gint exit = 1;
-          GString *cmd = expand_action (action + 1);
+          cmd = expand_action (action + 1);
           exit = run_command_sync (cmd->str, &data);
           if (exit == 0)
-            {
-              guint i = 0;
-              gchar **lines = g_strsplit (data, "\n", 0);
-              while (lines[i] && lines[i][0])
-                {
-                  gint fn;
-                  gchar *ptr = lines[i];
-
-                  while (isblank (*ptr)) ptr++;
-
-                  if (isdigit (*ptr))
-                    {
-                      gchar **ln = g_strsplit (ptr, ":", 2);
-                      fn = g_ascii_strtoll (ln[0], NULL, 10);
-                      if (fn && ln[1])
-                        set_field_value (fn - 1, ln[1]);
-                      g_strfreev (ln);
-                    }
-                  i++;
-                }
-            }
+            parse_cmd_output (data);
           g_free (data);
-          g_string_free (cmd, TRUE);
         }
       else
         {
-          GString *cmd = expand_action (action);
+          cmd = expand_action (action);
           run_command_async (cmd->str);
-          g_string_free (cmd, TRUE);
         }
+      g_string_free (cmd, TRUE);
     }
 
   /* set focus to specified field */
@@ -420,6 +431,33 @@ button_clicked_cb (GtkButton * b, gpointer d)
     gtk_widget_grab_focus (GTK_WIDGET (g_slist_nth_data (fields, options.form_data.focus_field - 1)));
 }
 
+static void
+field_changed_cb (GtkWidget *w, guint fn)
+{
+  gchar *data = NULL;
+  gint exit = 1;
+
+  if (disable_changed)
+    return;
+
+  if (options.form_data.changed_action)
+    {
+      gchar *str;
+      GString *cmd;
+
+      str = g_strdup_printf ("%s %d %%%d", options.form_data.changed_action, fn + 1, fn + 1);
+      cmd = expand_action (str);
+      g_free (str);
+
+      exit = run_command_sync (cmd->str, &data);
+      if (exit == 0)
+        parse_cmd_output (data);
+      g_free (data);
+
+      g_string_free (cmd, TRUE);
+    }
+}
+
 static void
 form_activate_cb (GtkEntry * entry, gpointer data)
 {
@@ -641,6 +679,44 @@ link_clicked_cb (GtkLinkButton *btn, gpointer data)
   return TRUE;
 }
 
+static void
+select_icon_cb (GtkEntry *e, GtkEntryIconPosition pos, GdkEventButton *ev, gpointer d)
+{
+  gchar *ib;
+
+  if (ev->button !=1 || pos != GTK_ENTRY_ICON_SECONDARY)
+    return;
+
+  ib = g_find_program_in_path ("yad-icon-browser");
+  if (ib)
+    {
+      gint exit;
+      gchar *cmd;
+      gchar *out = NULL;
+
+      cmd = g_strdup_printf ("%s -i", ib);
+      g_free (ib);
+
+      exit = run_command_sync (cmd, &out);
+      g_free (cmd);
+      if (exit == 0)
+        {
+          strip_new_line (out);
+          gtk_entry_set_text (e, out);
+          g_free (out);
+        }
+    }
+}
+
+static void
+set_icon_cb (GtkEntry *e, gpointer d)
+{
+  const gchar *icon = gtk_entry_get_text (e);
+
+  if (icon && *icon)
+    gtk_entry_set_icon_from_icon_name (e, GTK_ENTRY_ICON_PRIMARY, icon);
+}
+
 static gboolean
 handle_stdin (GIOChannel * ch, GIOCondition cond, gpointer data)
 {
@@ -662,10 +738,7 @@ handle_stdin (GIOChannel * ch, GIOCondition cond, gpointer data)
               if (options.form_data.cycle_read)
                 cnt = 0;
               else
-                {
-                  g_io_channel_shutdown (ch, TRUE, NULL);
-                  return FALSE;
-                }
+                goto shutdown;
             }
 
           do
@@ -686,8 +759,7 @@ handle_stdin (GIOChannel * ch, GIOCondition cond, gpointer data)
                   err = NULL;
                 }
               /* stop handling */
-              g_io_channel_shutdown (ch, TRUE, NULL);
-              return FALSE;
+              goto shutdown;
             }
 
           strip_new_line (string->str);
@@ -711,12 +783,14 @@ handle_stdin (GIOChannel * ch, GIOCondition cond, gpointer data)
     }
 
   if ((cond != G_IO_IN) && (cond != G_IO_IN + G_IO_HUP))
-    {
-      g_io_channel_shutdown (ch, TRUE, NULL);
-      return FALSE;
-    }
+    goto shutdown;
 
   return TRUE;
+
+ shutdown:
+  g_io_channel_shutdown (ch, TRUE, NULL);
+  disable_changed = FALSE;
+  return FALSE;
 }
 
 GtkWidget *
@@ -742,6 +816,8 @@ form_create_widget (GtkWidget * dlg)
       gtk_grid_set_row_spacing (GTK_GRID (tbl), 5);
       gtk_grid_set_column_spacing (GTK_GRID (tbl), 5);
 
+      gtk_grid_set_row_homogeneous (GTK_GRID (tbl), options.form_data.homogeneous);
+
       if (options.form_data.scroll)
         {
           GtkWidget *sw = gtk_scrolled_window_new (NULL, NULL);
@@ -797,6 +873,7 @@ form_create_widget (GtkWidget * dlg)
             case YAD_FIELD_HIDDEN:
             case YAD_FIELD_READ_ONLY:
             case YAD_FIELD_COMPLETE:
+            case YAD_FIELD_ICON:
               e = gtk_entry_new ();
               gtk_widget_set_name (e, "yad-form-entry");
               if (fld->tip)
@@ -829,6 +906,13 @@ form_create_widget (GtkWidget * dlg)
                   g_object_unref (m);
                   g_object_unref (c);
                 }
+              else if (fld->type == YAD_FIELD_ICON)
+                {
+                  gtk_entry_set_icon_from_icon_name (GTK_ENTRY (e), GTK_ENTRY_ICON_PRIMARY, "image-missing");
+                  gtk_entry_set_icon_from_icon_name (GTK_ENTRY (e), GTK_ENTRY_ICON_SECONDARY, "insert-image");
+                  g_signal_connect (G_OBJECT (e), "changed", G_CALLBACK (set_icon_cb), NULL);
+                  g_signal_connect (G_OBJECT (e), "icon-press", G_CALLBACK (select_icon_cb), NULL);
+                }
 
               gtk_label_set_mnemonic_widget (GTK_LABEL (l), e);
               fields = g_slist_append (fields, e);
@@ -871,6 +955,7 @@ form_create_widget (GtkWidget * dlg)
                 gtk_widget_set_hexpand (e, TRUE);
                 fields = g_slist_append (fields, e);
                 g_free (buf);
+                g_signal_connect_after (G_OBJECT (e), "toggled", G_CALLBACK (field_changed_cb), GINT_TO_POINTER (i));
               }
               break;
 
@@ -888,6 +973,7 @@ form_create_widget (GtkWidget * dlg)
               gtk_widget_set_hexpand (e, TRUE);
               gtk_label_set_mnemonic_widget (GTK_LABEL (l), e);
               fields = g_slist_append (fields, e);
+              g_signal_connect_after (G_OBJECT (e), "changed", G_CALLBACK (field_changed_cb), GINT_TO_POINTER (i));
               break;
 
             case YAD_FIELD_COMBO_ENTRY:
@@ -1090,7 +1176,10 @@ form_create_widget (GtkWidget * dlg)
                     gtk_widget_set_tooltip_text (e, fld->tip);
                 }
               g_signal_connect (G_OBJECT (e), "clicked", G_CALLBACK (button_clicked_cb), NULL);
-              gtk_container_add (GTK_CONTAINER (e), get_label (fld->name, 2, e));
+              l = get_label (fld->name, 2, e);
+              gtk_container_add (GTK_CONTAINER (e), l);
+              if (options.form_data.align_buttons)
+                gtk_widget_set_halign (l, options.common_data.align);
               if (fld->type == YAD_FIELD_BUTTON)
                 gtk_button_set_relief (GTK_BUTTON (e), GTK_RELIEF_NONE);
               gtk_grid_attach (GTK_GRID (tbl), e, col * 2, row, 2, 1);
@@ -1242,6 +1331,8 @@ form_create_widget (GtkWidget * dlg)
   if (options.form_data.focus_field > 0 && options.form_data.focus_field <= n_fields)
     gtk_widget_grab_focus (GTK_WIDGET (g_slist_nth_data (fields, options.form_data.focus_field - 1)));
 
+  disable_changed = FALSE;
+
   return w;
 }
 
@@ -1262,6 +1353,7 @@ form_print_field (guint fn)
     case YAD_FIELD_FILE_SAVE:
     case YAD_FIELD_DIR_CREATE:
     case YAD_FIELD_DATE:
+    case YAD_FIELD_ICON:
       if (options.common_data.quoted_output)
         {
           buf = g_shell_quote (gtk_entry_get_text (GTK_ENTRY (g_slist_nth_data (fields, fn))));
