@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2021, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <limits.h>
@@ -25,6 +25,8 @@
 #include <webkit2/webkit2.h>
 
 static WebKitWebView *view;
+static WebKitFindController *find_ctl = NULL;;
+static YadSearchBar *search_bar = NULL;
 
 static GString *inbuf;
 
@@ -34,6 +36,49 @@ static gboolean is_loaded = FALSE;
 #define PATH_MAX 4096
 #endif
 
+/* searching */
+static void
+do_find_next (GtkWidget *w, gpointer d)
+{
+  webkit_find_controller_search_next (find_ctl);
+}
+
+static void
+do_find_prev (GtkWidget *w, gpointer d)
+{
+  webkit_find_controller_search_previous (find_ctl);
+}
+
+static void
+search_changed_cb (GtkWidget *w, gpointer d)
+{
+  WebKitFindOptions fopts = WEBKIT_FIND_OPTIONS_WRAP_AROUND;
+
+  if (!find_ctl)
+    find_ctl = webkit_web_view_get_find_controller (view);
+  search_bar->new_search = TRUE;
+  search_bar->str = gtk_entry_get_text (GTK_ENTRY (search_bar->entry));
+
+  if (!search_bar->case_sensitive)
+    fopts |= WEBKIT_FIND_OPTIONS_CASE_INSENSITIVE;
+
+  webkit_find_controller_search (find_ctl, search_bar->str, fopts, G_MAXUINT);
+}
+
+static void
+stop_search_cb (GtkWidget *w, YadSearchBar *sb)
+{
+  if (find_ctl)
+    {
+      webkit_find_controller_search_finish (find_ctl);
+      find_ctl = NULL;
+    }
+
+  gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (search_bar->bar), FALSE);
+  gtk_widget_grab_focus (GTK_WIDGET (view));
+  ignore_esc = FALSE;
+}
+
 static void
 load_uri (const gchar * uri)
 {
@@ -171,8 +216,8 @@ select_file_cb (GtkEntry *entry, GtkEntryIconPosition pos, GdkEventButton *ev, g
   dlg = gtk_file_chooser_dialog_new (_("YAD - Select File"),
                                      GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (entry))),
                                      GTK_FILE_CHOOSER_ACTION_OPEN,
-                                     _("OK"), GTK_RESPONSE_ACCEPT,
                                      _("Cancel"), GTK_RESPONSE_CANCEL,
+                                     _("OK"), GTK_RESPONSE_ACCEPT,
                                      NULL);
   if (dir)
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), dir);
@@ -202,8 +247,7 @@ open_cb (GSimpleAction *act, GVariant *param, gpointer d)
 {
   GtkWidget *dlg, *cnt, *lbl, *entry;
 
-  dlg = gtk_dialog_new_with_buttons (_("Open URI"),
-                                     GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
+  dlg = gtk_dialog_new_with_buttons (_("Open URI"), GTK_WINDOW (d),
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
                                      _("Cancel"), GTK_RESPONSE_REJECT,
                                      _("Open"), GTK_RESPONSE_ACCEPT,
@@ -247,7 +291,7 @@ menu_cb (WebKitWebView *view, WebKitContextMenu *menu, GdkEvent *ev, WebKitHitTe
   webkit_context_menu_prepend (menu, mi);
 
   act = g_simple_action_new ("open", NULL);
-  g_signal_connect (G_OBJECT (act), "activate", G_CALLBACK (open_cb), NULL);
+  g_signal_connect (G_OBJECT (act), "activate", G_CALLBACK (open_cb), d);
 
   mi = webkit_context_menu_item_new_from_gaction (G_ACTION (act), _("Open URI"), NULL);
   webkit_context_menu_prepend (menu, mi);
@@ -256,7 +300,7 @@ menu_cb (WebKitWebView *view, WebKitContextMenu *menu, GdkEvent *ev, WebKitHitTe
   webkit_context_menu_append (menu, mi);
 
   act = g_simple_action_new ("quit", NULL);
-  g_signal_connect (G_OBJECT (act), "activate", G_CALLBACK (quit_cb), NULL);
+  g_signal_connect (G_OBJECT (act), "activate", G_CALLBACK (quit_cb), d);
 
   mi = webkit_context_menu_item_new_from_gaction (G_ACTION (act), _("Quit"), NULL);
   webkit_context_menu_append (menu, mi);
@@ -264,6 +308,32 @@ menu_cb (WebKitWebView *view, WebKitContextMenu *menu, GdkEvent *ev, WebKitHitTe
   return FALSE;
 }
 
+static gboolean
+key_press_cb (GtkWidget *w, GdkEventKey *key, gpointer d)
+{
+  if ((key->state & GDK_CONTROL_MASK) && (key->keyval == GDK_KEY_O || key->keyval == GDK_KEY_o))
+    {
+      open_cb (NULL, NULL, d);
+      return TRUE;
+    }
+  else if ((key->state & GDK_CONTROL_MASK) && (key->keyval == GDK_KEY_Q || key->keyval == GDK_KEY_q))
+    {
+      yad_exit (options.data.def_resp);
+      return TRUE;
+    }
+  else if ((key->state & GDK_CONTROL_MASK) && (key->keyval == GDK_KEY_F || key->keyval == GDK_KEY_f))
+    {
+      if (search_bar == NULL)
+        return FALSE;
+
+      ignore_esc = TRUE;
+      gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (search_bar->bar), TRUE);
+      return gtk_search_bar_handle_event (GTK_SEARCH_BAR (search_bar->bar), (GdkEvent *) key);
+    }
+
+  return FALSE;
+}
+
 static void
 title_cb (GObject *obj, GParamSpec *spec, GtkWindow *dlg)
 {
@@ -316,14 +386,52 @@ handle_stdin (GIOChannel * ch, GIOCondition cond, gpointer d)
   return FALSE;
 }
 
+static void
+set_user_props (WebKitSettings *wk_settings)
+{
+  gint i;
+
+  if (!options.html_data.wk_props)
+    return;
+
+  for (i = 0; options.html_data.wk_props[i] != NULL; i++)
+    {
+      gchar **prop = g_strsplit (options.html_data.wk_props[i], " ", 2);
+
+      if (prop[0] && prop[1])
+        {
+          switch (prop[1][0])
+            {
+            case 'b':
+              g_object_set (G_OBJECT (wk_settings), prop[0], (strcmp (prop[1] + 2, "true") == 0), NULL);
+              break;
+            case 'i':
+              g_object_set (G_OBJECT (wk_settings), prop[0], atol (prop[1] + 2), NULL);
+              break;
+            case 's':
+              g_object_set (G_OBJECT (wk_settings), prop[0], prop[1] + 2, NULL);
+              break;
+            default:
+              if (options.debug)
+                g_warning ("Wrong value '%s' for setting '%s'\n", prop[1], prop[0]);
+            }
+        }
+
+      g_strfreev (prop);
+    }
+}
+
 GtkWidget *
 html_create_widget (GtkWidget * dlg)
 {
-  GtkWidget *sw;
+  GtkWidget *w, *sw;
   WebKitSettings *wk_settings;
 
+  w = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+
   sw = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), options.hscroll_policy, options.vscroll_policy);
+  gtk_box_pack_start (GTK_BOX (w), sw, TRUE, TRUE, 0);
 
   view = WEBKIT_WEB_VIEW (webkit_web_view_new ());
   gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (view));
@@ -345,7 +453,8 @@ html_create_widget (GtkWidget * dlg)
 
   if (options.html_data.browser)
     {
-      g_signal_connect (view, "context-menu", G_CALLBACK (menu_cb), NULL);
+      g_signal_connect (view, "context-menu", G_CALLBACK (menu_cb), dlg);
+      g_signal_connect (view, "key-press-event", G_CALLBACK (key_press_cb), dlg);
       if (!options.data.dialog_title)
         g_signal_connect (view, "notify::title", G_CALLBACK (title_cb), dlg);
       if (!options.data.window_icon)
@@ -353,19 +462,37 @@ html_create_widget (GtkWidget * dlg)
     }
   else
     {
-      g_object_set (G_OBJECT (wk_settings), "enable-caret-browsing", FALSE, NULL);
-      g_object_set (G_OBJECT (wk_settings), "enable-developer-extras", FALSE, NULL);
-      g_object_set (G_OBJECT (wk_settings), "enable-html5-database", FALSE, NULL);
-      g_object_set (G_OBJECT (wk_settings), "enable-html5-local-storage", FALSE, NULL);
-      g_object_set (G_OBJECT (wk_settings), "enable-offline-web-application-cache", FALSE, NULL);
-      g_object_set (G_OBJECT (wk_settings), "enable-page-cache", FALSE, NULL);
-      g_object_set (G_OBJECT (wk_settings), "enable-plugins", FALSE, NULL);
-      g_object_set (G_OBJECT (wk_settings), "enable-private-browsing", TRUE, NULL);
+      g_object_set (G_OBJECT (wk_settings),
+                    "enable-write-console-messages-to-stdout", TRUE,
+                    "enable-caret-browsing", FALSE,
+                    "enable-developer-extras", FALSE,
+                    "enable-html5-database", FALSE,
+                    "enable-html5-local-storage", FALSE,
+                    "enable-offline-web-application-cache", FALSE,
+                    "enable-page-cache", FALSE,
+                    "enable-plugins", FALSE,
+                    NULL);
     }
 
+  set_user_props (wk_settings);
+
   gtk_widget_show_all (sw);
   gtk_widget_grab_focus (GTK_WIDGET (view));
 
+  /* create search bar */
+  if (options.common_data.enable_search)
+    {
+      if ((search_bar = create_search_bar ()) != NULL)
+        {
+          gtk_box_pack_start (GTK_BOX (w), search_bar->bar, FALSE, FALSE, 0);
+          g_signal_connect (G_OBJECT (search_bar->entry), "search-changed", G_CALLBACK (search_changed_cb), NULL);
+          g_signal_connect (G_OBJECT (search_bar->entry), "stop-search", G_CALLBACK (stop_search_cb), NULL);
+          g_signal_connect (G_OBJECT (search_bar->entry), "next-match", G_CALLBACK (do_find_next), NULL);
+          g_signal_connect (G_OBJECT (search_bar->entry), "previous-match", G_CALLBACK (do_find_prev), NULL);
+        }
+    }
+
+  /* load data */
   if (options.html_data.uri)
     load_uri (options.html_data.uri);
   else if (!options.html_data.browser)
@@ -381,5 +508,5 @@ html_create_widget (GtkWidget * dlg)
   else if (options.extra_data)
     load_uri (options.extra_data[0]);
 
-  return sw;
+  return w;
 }
