@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2021, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <sys/types.h>
@@ -38,11 +38,15 @@ GtkIconTheme *yad_icon_theme;
 
 #ifndef STANDALONE
 GSettings *settings;
+GSettings *sv_settings;
 #endif
 
 GdkPixbuf *big_fallback_image = NULL;
 GdkPixbuf *small_fallback_image = NULL;
 
+/* global flag for temporary disable ESC handling (needed for searchbar) */
+gboolean ignore_esc = FALSE;
+
 static GtkWidget *dialog = NULL;
 static GtkWidget *text = NULL;
 
@@ -80,12 +84,23 @@ keys_cb (GtkWidget *w, GdkEventKey *ev, gpointer d)
 
   switch (ev->keyval)
     {
+    case GDK_KEY_F1:
+      if (options.data.f1_action)
+        {
+          run_command_async (options.data.f1_action);
+          return TRUE;
+        }
+      break;
     case GDK_KEY_Escape:
-      if (options.data.escape_ok)
-          yad_exit (options.data.def_resp);
-      else if (!options.data.no_escape)
-         yad_exit (YAD_RESPONSE_ESC);
-      return TRUE;
+      if (!ignore_esc)
+        {
+          if (options.data.escape_ok)
+            yad_exit (options.data.def_resp);
+          else if (!options.data.no_escape)
+            yad_exit (YAD_RESPONSE_ESC);
+          return TRUE;
+        }
+      break;
     case GDK_KEY_Return:
     case GDK_KEY_KP_Enter:
       if (ev->state & GDK_CONTROL_MASK)
@@ -93,7 +108,9 @@ keys_cb (GtkWidget *w, GdkEventKey *ev, gpointer d)
           yad_exit (options.data.def_resp);
           return TRUE;
         }
+      break;
      }
+
    return FALSE;
 }
 
@@ -317,7 +334,12 @@ create_layout (GtkWidget *dlg)
 
   gtk_box_pack_start (GTK_BOX (layout), box, FALSE, FALSE, 0);
   if (imw)
-    gtk_box_pack_start (GTK_BOX (layout), imw, TRUE, TRUE, 0);
+    {
+      if (options.mode == YAD_MODE_ENTRY)
+        gtk_box_pack_start (GTK_BOX (layout), imw, TRUE, FALSE, 0);
+      else
+        gtk_box_pack_start (GTK_BOX (layout), imw, TRUE, TRUE, 0);
+    }
 
   if (options.mode == YAD_MODE_DND)
     dnd_init (layout);
@@ -697,6 +719,7 @@ main (gint argc, gchar ** argv)
 
 #ifndef STANDALONE
   settings = g_settings_new ("yad.settings");
+  sv_settings = g_settings_new ("yad.sourceview");
 #endif
 
   yad_icon_theme = gtk_icon_theme_get_default ();
@@ -868,18 +891,6 @@ main (gint argc, gchar ** argv)
       g_print ("%s (GTK+ %d.%d.%d)\n", VERSION, gtk_major_version, gtk_minor_version, gtk_micro_version);
       break;
 
-#ifdef HAVE_SPELL
-    case YAD_MODE_LANGS:
-      show_langs ();
-      break;
-#endif
-
-#ifdef HAVE_SOURCEVIEW
-    case YAD_MODE_THEMES:
-      show_themes ();
-      break;
-#endif
-
 #ifdef HAVE_TRAY
     case YAD_MODE_NOTIFICATION:
       ret = yad_notification_run ();
