@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2021, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <stdlib.h>
@@ -25,10 +25,10 @@ static gboolean add_button (const gchar *, const gchar *, gpointer, GError **);
 static gboolean add_column (const gchar *, const gchar *, gpointer, GError **);
 static gboolean add_field (const gchar *, const gchar *, gpointer, GError **);
 static gboolean add_bar (const gchar *, const gchar *, gpointer, GError **);
-static gboolean add_tab (const gchar *, const gchar *, gpointer, GError **);
 static gboolean add_scale_mark (const gchar *, const gchar *, gpointer, GError **);
 static gboolean add_palette (const gchar *, const gchar *, gpointer, GError **);
 static gboolean add_confirm_overwrite (const gchar *, const gchar *, gpointer, GError **);
+static gboolean add_confirm_save (const gchar *, const gchar *, gpointer, GError **);
 static gboolean add_file_filter (const gchar *, const gchar *, gpointer, GError **);
 static gboolean set_color_mode (const gchar *, const gchar *, gpointer, GError **);
 static gboolean set_buttons_layout (const gchar *, const gchar *, gpointer, GError **);
@@ -57,15 +57,13 @@ static gboolean set_scroll_policy (const gchar *, const gchar *, gpointer, GErro
 static gboolean set_size_format (const gchar *, const gchar *, gpointer, GError **);
 #endif
 static gboolean set_interp (const gchar *, const gchar *, gpointer, GError **);
+#ifdef HAVE_SOURCEVIEW
+static gboolean set_right_margin (const gchar *, const gchar *, gpointer, GError **);
+static gboolean set_smart_homend (const gchar *, const gchar *, gpointer, GError **);
+#endif
 
 static gboolean about_mode = FALSE;
 static gboolean version_mode = FALSE;
-#ifdef HAVE_SPELL
-static gboolean langs_mode = FALSE;
-#endif
-#ifdef HAVE_SOURCEVIEW
-static gboolean themes_mode = FALSE;
-#endif
 static gboolean app_mode = FALSE;
 static gboolean calendar_mode = FALSE;
 static gboolean color_mode = FALSE;
@@ -147,6 +145,8 @@ static GOptionEntry general_options[] = {
     N_("Run commands under specified interpreter (default: bash -c '%s')"), N_("CMD") },
   { "uri-handler", 0, 0, G_OPTION_ARG_STRING, &options.data.uri_handler,
     N_("Set URI handler"), N_("CMD") },
+  { "f1-action", 0, 0, G_OPTION_ARG_STRING, &options.data.f1_action,
+    N_("Set command running when F1 was pressed"), N_("CMD") },
   /* window settings */
   { "sticky", 0, 0, G_OPTION_ARG_NONE, &options.data.sticky,
     N_("Set window sticky"), NULL },
@@ -226,6 +226,8 @@ static GOptionEntry common_options[] = {
     N_("Set extended completion for entries (any, all, or regex)"), N_("TYPE") },
   { "bool-fmt", 0, 0, G_OPTION_ARG_CALLBACK, set_bool_fmt_type,
     N_("Set type of output for boolean values (T, t, Y, y, O, o, 1)"), N_("TYPE") },
+  { "disable-search", 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &options.common_data.enable_search,
+    N_("Disable search in text and html dialogs"), NULL },
 #if GLIB_CHECK_VERSION(2,30,0)
   { "iec-format", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, set_size_format,
     N_("Use IEC (base 1024) units with for size values"), NULL },
@@ -239,6 +241,28 @@ static GOptionEntry common_options[] = {
   { NULL }
 };
 
+static GOptionEntry about_options[] = {
+  { "about", 0, G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &about_mode,
+    N_("Display about dialog"), NULL },
+  { "pname", 0, 0, G_OPTION_ARG_STRING, &options.about_data.name,
+    N_("Set application name"), N_("STRING") },
+  { "pversion", 0, 0, G_OPTION_ARG_STRING, &options.about_data.version,
+    N_("Set application version"), N_("STRING") },
+  { "copyright", 0, 0, G_OPTION_ARG_STRING, &options.about_data.copyright,
+    N_("Set application copyright string"), N_("STRING") },
+  { "comments", 0, 0, G_OPTION_ARG_STRING, &options.about_data.comments,
+    N_("Set application comments string"), N_("TEXT") },
+  { "license", 0, 0, G_OPTION_ARG_STRING, &options.about_data.license,
+    N_("Set application license"), N_("TEXT") },
+  { "authors", 0, 0, G_OPTION_ARG_STRING, &options.about_data.authors,
+    N_("Set application authors"), N_("LIST") },
+  { "website", 0, 0, G_OPTION_ARG_STRING, &options.about_data.website,
+    N_("Set application website"), N_("URI") },
+  { "website-label", 0, 0, G_OPTION_ARG_STRING, &options.about_data.website_lbl,
+    N_("Set application website label"), N_("STRING") },
+  { NULL }
+};
+
 static GOptionEntry app_options[] = {
   { "app", 0, G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &app_mode,
     N_("Display application selection dialog"), NULL },
@@ -359,6 +383,10 @@ static GOptionEntry form_options[] = {
     N_("Set focused field"), N_("NUMBER") },
   { "cycle-read", 0, 0, G_OPTION_ARG_NONE, &options.form_data.cycle_read,
     N_("Cycled reading of stdin data"), NULL },
+  { "align-buttons", 0, 0, G_OPTION_ARG_NONE, &options.form_data.align_buttons,
+    N_("Align labels on button fields"), NULL },
+  { "changed-action", 0, 0, G_OPTION_ARG_STRING, &options.form_data.changed_action,
+    N_("Set changed action"), "CMD" },
   { NULL }
 };
 
@@ -380,6 +408,8 @@ static GOptionEntry html_options[] = {
     N_("Set user agent string"), N_("STRING") },
   { "user-style", 0, 0, G_OPTION_ARG_STRING, &options.html_data.user_style,
     N_("Set path or uri to user styles"), "URI" },
+  { "wk-prop", 0, 0, G_OPTION_ARG_STRING_ARRAY, &options.html_data.wk_props,
+    N_("Set WebKit property"), N_("PROP") },
   { NULL }
 };
 #endif
@@ -480,7 +510,7 @@ static GOptionEntry list_options[] = {
 static GOptionEntry notebook_options[] = {
   { "notebook", 0, G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &notebook_mode,
     N_("Display notebook dialog"), NULL },
-  { "tab", 0, 0, G_OPTION_ARG_CALLBACK, add_tab,
+  { "tab", 0, 0, G_OPTION_ARG_STRING_ARRAY, &options.notebook_data.tabs,
     N_("Add a tab to notebook"), N_("LABEL") },
   { "tab-pos", 0, G_OPTION_FLAG_NOALIAS, G_OPTION_ARG_CALLBACK, set_tab_pos,
     N_("Set position of a notebook tabs (top, bottom, left or right)"), N_("TYPE") },
@@ -490,6 +520,8 @@ static GOptionEntry notebook_options[] = {
     N_("Set active tab"), N_("NUMBER") },
   { "expand", 0, 0, G_OPTION_ARG_NONE, &options.notebook_data.expand,
     N_("Expand tabs"), NULL },
+  { "stack", 0, 0, G_OPTION_ARG_NONE, &options.notebook_data.stack,
+    N_("Use stack mode"), NULL },
   { NULL }
 };
 
@@ -516,6 +548,8 @@ static GOptionEntry paned_options[] = {
     N_("Set orientation (hor[izontal] or vert[ical])"), N_("TYPE") },
   { "splitter", 0, 0, G_OPTION_ARG_INT, &options.paned_data.splitter,
     N_("Set initial splitter position"), N_("POS") },
+  { "focused", 0, 0, G_OPTION_ARG_INT, &options.paned_data.focused,
+    N_("Set focused pane (1 or 2)"), N_("PANE") },
   { NULL }
 };
 
@@ -623,6 +657,12 @@ static GOptionEntry text_options[] = {
     N_("Use specified color for links"), N_("COLOR") },
   { "formatted", 0, 0, G_OPTION_ARG_NONE, &options.text_data.formatted,
     N_("Use pango markup"), NULL },
+  { "in-place", 0, 0, G_OPTION_ARG_NONE, &options.text_data.in_place,
+    N_("Save file instead of print on exit"), NULL },
+  { "file-op", 0, 0, G_OPTION_ARG_NONE, &options.text_data.file_op,
+    N_("Enable file operations"), NULL },
+  { "confirm-save", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, add_confirm_save,
+    N_("Confirm save the file if text was changed"), N_("[TEXT]") },
   { NULL }
 };
 
@@ -632,6 +672,32 @@ static GOptionEntry source_options[] = {
     N_("Use specified langauge for syntax highlighting"), N_("LANG") },
   { "theme", 0, 0, G_OPTION_ARG_STRING, &options.source_data.theme,
     N_("Use specified theme"), N_("THEME") },
+  { "line-num", 0, 0, G_OPTION_ARG_NONE, &options.source_data.line_num,
+    N_("Show line numbers"), NULL },
+  { "line-hl", 0, 0, G_OPTION_ARG_NONE, &options.source_data.line_hl,
+    N_("Highlight current line"), NULL },
+  { "line-marks", 0, 0, G_OPTION_ARG_NONE, &options.source_data.line_marks,
+    N_("Enable line marks"), NULL },
+  { "mark1-color", 0, 0, G_OPTION_ARG_STRING, &options.source_data.m1_color,
+    N_("Set color for first type marks"), N_("COLOR") },
+  { "mark2-color", 0, 0, G_OPTION_ARG_STRING, &options.source_data.m1_color,
+    N_("Set color for second type marks"), N_("COLOR") },
+  { "right-margin", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, set_right_margin,
+    N_("Set position of right margin"), N_("[POS]") },
+  { "brackets", 0, 0, G_OPTION_ARG_NONE, &options.source_data.brackets,
+    N_("Highlight matching brackets"), NULL },
+  { "indent", 0, 0, G_OPTION_ARG_NONE, &options.source_data.indent,
+    N_("Use autoindent"), NULL },
+  { "tab-width", 0, 0, G_OPTION_ARG_INT, &options.source_data.tab_width,
+    N_("Set tabulation width"), N_("VALUE") },
+  { "indent-width", 0, 0, G_OPTION_ARG_INT, &options.source_data.indent_width,
+    N_("Set indentation width"), N_("VALUE") },
+  { "smart-he", 0, 0, G_OPTION_ARG_CALLBACK, set_smart_homend,
+    N_("Set smart Home/End behavior"), N_("TYPE") },
+  { "smart-bs", 0, 0, G_OPTION_ARG_NONE, &options.source_data.smart_bs,
+    N_("Enable smart backspace"), NULL },
+  { "spaces", 0, 0, G_OPTION_ARG_NONE, &options.source_data.spaces,
+    N_("Insert spaces instead of tabs"), NULL },
   { NULL }
 };
 #endif
@@ -647,18 +713,8 @@ static GOptionEntry filter_options[] = {
 };
 
 static GOptionEntry misc_options[] = {
-  { "about", 0, 0, G_OPTION_ARG_NONE, &about_mode,
-    N_("Show about dialog"), NULL },
   { "version", 0, 0, G_OPTION_ARG_NONE, &version_mode,
     N_("Print version"), NULL },
-#ifdef HAVE_SPELL
-  { "show-langs", 0, 0, G_OPTION_ARG_NONE, &langs_mode,
-    N_("Show list of spell languages"), NULL },
-#endif
-#ifdef HAVE_SOURCEVIEW
-  { "show-themes", 0, 0, G_OPTION_ARG_NONE, &themes_mode,
-    N_("Show list of GtkSourceView themes"), NULL },
-#endif
   { "css", 0, 0, G_OPTION_ARG_STRING, &options.css,
     N_("Load additional CSS settings from file or string"), N_("STRING") },
   { "gtkrc", 0, 0, G_OPTION_ARG_FILENAME, &options.gtkrc_file,
@@ -806,6 +862,8 @@ add_field (const gchar * option_name, const gchar * value, gpointer data, GError
         fld->type = YAD_FIELD_FONT;
       else if (strcasecmp (fstr[1], "APP") == 0)
         fld->type = YAD_FIELD_APP;
+      else if (strcasecmp (fstr[1], "ICON") == 0)
+        fld->type = YAD_FIELD_ICON;
       else if (strcasecmp (fstr[1], "CLR") == 0)
         fld->type = YAD_FIELD_COLOR;
       else if (strcasecmp (fstr[1], "MFL") == 0)
@@ -833,6 +891,10 @@ add_field (const gchar * option_name, const gchar * value, gpointer data, GError
     fld->type = YAD_FIELD_SIMPLE;
   options.form_data.fields = g_slist_append (options.form_data.fields, fld);
 
+  /* disable homogeneous rows if form has text field */
+  if (fld->type == YAD_FIELD_TEXT)
+    options.form_data.homogeneous = FALSE;
+
   g_strfreev (fstr);
   return TRUE;
 }
@@ -864,13 +926,6 @@ add_bar (const gchar * option_name, const gchar * value, gpointer data, GError *
   return TRUE;
 }
 
-static gboolean
-add_tab (const gchar * option_name, const gchar * value, gpointer data, GError ** err)
-{
-  options.notebook_data.tabs = g_slist_append (options.notebook_data.tabs, g_strdup (value));
-  return TRUE;
-}
-
 static gboolean
 add_scale_mark (const gchar * option_name, const gchar * value, gpointer data, GError ** err)
 {
@@ -915,6 +970,16 @@ add_confirm_overwrite (const gchar * option_name, const gchar * value, gpointer
   return TRUE;
 }
 
+static gboolean
+add_confirm_save (const gchar * option_name, const gchar * value, gpointer data, GError ** err)
+{
+  options.text_data.confirm_save = TRUE;
+  if (value)
+    options.text_data.confirm_text = g_strdup (value);
+
+  return TRUE;
+}
+
 static gboolean
 add_file_filter (const gchar * option_name, const gchar * value, gpointer data, GError ** err)
 {
@@ -1290,6 +1355,34 @@ set_interp (const gchar * option_name, const gchar * value, gpointer data, GErro
   return TRUE;
 }
 
+#if HAVE_SOURCEVIEW
+static gboolean
+set_right_margin (const gchar * option_name, const gchar * value, gpointer data, GError ** err)
+{
+  if (value)
+    options.source_data.right_margin = atoi (value);
+  else
+    options.source_data.right_margin = RIGHT_MARGIN;
+}
+
+static gboolean
+set_smart_homend (const gchar * option_name, const gchar * value, gpointer data, GError ** err)
+{
+  if (strcasecmp (value, "never") == 0)
+    options.source_data.smart_he = GTK_SOURCE_SMART_HOME_END_DISABLED;
+  else if (strcasecmp (value, "before") == 0)
+    options.source_data.smart_he = GTK_SOURCE_SMART_HOME_END_BEFORE;
+  else if (strcasecmp (value, "after") == 0)
+    options.source_data.smart_he = GTK_SOURCE_SMART_HOME_END_AFTER;
+  else if (strcasecmp (value, "always") == 0)
+    options.source_data.smart_he = GTK_SOURCE_SMART_HOME_END_ALWAYS;
+  else
+    g_printerr (_("Unknown smart Home/End mode: %s\n"), value);
+
+  return TRUE;
+}
+#endif
+
 #ifndef G_OS_WIN32
 static gboolean
 set_xid_file (const gchar * option_name, const gchar * value, gpointer data, GError ** err)
@@ -1449,14 +1542,6 @@ yad_set_mode (void)
     options.mode = YAD_MODE_ABOUT;
   else if (version_mode)
     options.mode = YAD_MODE_VERSION;
-#ifdef HAVE_SPELL
-  else if (langs_mode)
-    options.mode = YAD_MODE_LANGS;
-#endif
-#ifdef HAVE_SOURCEVIEW
-  else if (themes_mode)
-    options.mode = YAD_MODE_THEMES;
-#endif
 }
 
 void
@@ -1525,12 +1610,13 @@ yad_options_init (void)
   options.data.keep_icon_size = FALSE;
   options.data.def_resp = YAD_RESPONSE_OK;
   options.data.use_interp = FALSE;
-  options.data.interp = "bash -c '%s'";
+  options.data.interp = "bash -c \"%s\"";
 #ifndef STANDALONE
   options.data.uri_handler = g_settings_get_string (settings, "open-command");
 #else
   options.data.uri_handler = OPEN_CMD;
 #endif
+  options.data.f1_action = NULL;
 
   /* Initialize window options */
   options.data.sticky = FALSE;
@@ -1579,6 +1665,7 @@ yad_options_init (void)
   options.common_data.bool_fmt = YAD_BOOL_FMT_UT;
   options.common_data.complete = YAD_COMPLETE_SIMPLE;
   options.common_data.icon_size = 0;
+  options.common_data.enable_search = TRUE;
 #if GLIB_CHECK_VERSION(2,30,0)
   options.common_data.size_fmt = G_FORMAT_SIZE_DEFAULT;
 #endif
@@ -1587,6 +1674,15 @@ yad_options_init (void)
   options.common_data.spell_lang = NULL;
 #endif
 
+  /* initialize about data */
+  options.about_data.name = NULL;
+  options.about_data.copyright = NULL;
+  options.about_data.comments = NULL;
+  options.about_data.license = NULL;
+  options.about_data.authors = NULL;
+  options.about_data.website = NULL;
+  options.about_data.website_lbl = NULL;
+
   /* Initialize application data */
   options.app_data.show_fallback = FALSE;
   options.app_data.show_other = FALSE;
@@ -1627,7 +1723,7 @@ yad_options_init (void)
   options.file_data.directory = FALSE;
   options.file_data.save = FALSE;
   options.file_data.confirm_overwrite = FALSE;
-  options.file_data.confirm_text = N_("File exist. Overwrite?");
+  options.file_data.confirm_text = _("File exist. Overwrite?");
   options.file_data.file_filt = NULL;
   options.file_data.mime_filt = NULL;
   options.file_data.image_filt = NULL;
@@ -1643,6 +1739,9 @@ yad_options_init (void)
   options.form_data.output_by_row = FALSE;
   options.form_data.focus_field = 1;
   options.form_data.cycle_read = FALSE;
+  options.form_data.align_buttons = FALSE;
+  options.form_data.changed_action = NULL;
+  options.form_data.homogeneous = TRUE;
 
 #ifdef HAVE_HTML
   /* Initialize html data */
@@ -1653,6 +1752,7 @@ yad_options_init (void)
   options.html_data.encoding = NULL;
   options.html_data.user_agent = "YAD-Webkit (" VERSION ")";
   options.html_data.user_style = NULL;
+  options.html_data.wk_props = NULL;
 #endif
 
   /* Initialize icons data */
@@ -1706,8 +1806,9 @@ yad_options_init (void)
   options.notebook_data.tabs = NULL;
   options.notebook_data.borders = 5;
   options.notebook_data.pos = GTK_POS_TOP;
-  options.notebook_data.expand = FALSE;
   options.notebook_data.active = 1;
+  options.notebook_data.expand = FALSE;
+  options.notebook_data.stack = FALSE;
 
 #ifdef HAVE_TRAY
   /* Initialize notification data */
@@ -1719,6 +1820,7 @@ yad_options_init (void)
   /* Initialize paned data */
   options.paned_data.orient = GTK_ORIENTATION_VERTICAL;
   options.paned_data.splitter = -1;
+  options.paned_data.focused = 1;
 
   /* Initialize picture data */
   options.picture_data.size = YAD_PICTURE_ORIG;
@@ -1764,17 +1866,51 @@ yad_options_init (void)
   options.text_data.hide_cursor = TRUE;
   options.text_data.fore = NULL;
   options.text_data.back = NULL;
+  options.text_data.confirm_save = FALSE;
+  options.text_data.confirm_text = _("File was changed. Save it?");
 #ifndef STANDALONE
   options.text_data.uri_color = g_settings_get_string (settings, "uri-color");
 #else
   options.text_data.uri_color = URI_COLOR;
 #endif
   options.text_data.formatted = FALSE;
+  options.text_data.in_place = FALSE;
+  options.text_data.file_op = FALSE;
 
 #ifdef HAVE_SOURCEVIEW
   /* Initialize sourceview data */
   options.source_data.lang = NULL;
+#ifndef STANDALONE
+  options.source_data.theme = g_settings_get_string (sv_settings, "theme");;
+  options.source_data.line_num = g_settings_get_boolean (sv_settings, "line-num");
+  options.source_data.line_hl = g_settings_get_boolean (sv_settings, "line-hl");
+  options.source_data.line_marks = g_settings_get_boolean (sv_settings, "line-marks");
+  options.source_data.m1_color = g_settings_get_string (sv_settings, "mark1-color");
+  options.source_data.m2_color = g_settings_get_string (sv_settings, "mark2-color");
+  options.source_data.right_margin = g_settings_get_int (sv_settings, "right-margin");
+  options.source_data.brackets = g_settings_get_boolean (sv_settings, "brackets");
+  options.source_data.indent = g_settings_get_boolean (sv_settings, "indent");
+  options.source_data.tab_width = g_settings_get_int (sv_settings, "tab-width");
+  options.source_data.indent_width = g_settings_get_int (sv_settings, "indent-width");
+  options.source_data.smart_he = g_settings_get_enum (sv_settings, "homend");
+  options.source_data.smart_bs = g_settings_get_boolean (sv_settings, "smart-bs");
+  options.source_data.spaces = g_settings_get_boolean (sv_settings, "spaces");
+#else
   options.source_data.theme = NULL;
+  options.source_data.line_num = FALSE;
+  options.source_data.line_hl = FALSE;
+  options.source_data.line_marks = FALSE;
+  options.source_data.m1_color = MARK1_COLOR;
+  options.source_data.m2_color = MARK2_COLOR;
+  options.source_data.right_margin = 0;
+  options.source_data.brackets = FALSE;
+  options.source_data.indent = FALSE;
+  options.source_data.tab_width = 8;
+  options.source_data.indent_width = 4;
+  options.source_data.smart_he = GTK_SOURCE_SMART_HOME_END_DISABLED;
+  options.source_data.smart_bs = FALSE;
+  options.source_data.spaces = FALSE;
+#endif
 #endif
 }
 
@@ -1799,6 +1935,12 @@ yad_create_context (void)
   g_option_group_set_translation_domain (a_group, GETTEXT_PACKAGE);
   g_option_context_add_group (tmp_ctx, a_group);
 
+  /* Adds about option entries */
+  a_group = g_option_group_new ("about", _("About dialog options"), _("Show custom about dialog options"), NULL, NULL);
+  g_option_group_add_entries (a_group, about_options);
+  g_option_group_set_translation_domain (a_group, GETTEXT_PACKAGE);
+  g_option_context_add_group (tmp_ctx, a_group);
+
   /* Adds app option entries */
   a_group = g_option_group_new ("app", _("Application selection options"), _("Show application selection options"), NULL, NULL);
   g_option_group_add_entries (a_group, app_options);
