@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2021, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <pango/pango.h>
@@ -25,135 +25,295 @@ static GtkWidget *text_view;
 static GObject *text_buffer;
 static GtkTextTag *tag;
 static GdkCursor *hand, *normal;
-static gchar *pattern = NULL;
-static gboolean new_search = TRUE;
+static YadSearchBar *search_bar = NULL;
+static GtkTextIter match_start, match_end;
+static gboolean text_changed = FALSE;
+static gboolean search_changed = FALSE;
 
 /* searching */
 static void
-do_search (GtkWidget * e, GtkWidget * w)
+do_find_next (GtkWidget *w, gpointer d)
 {
-  static gchar *text = NULL;
-  static guint offset;
-  static GRegex *regex = NULL;
-  GMatchInfo *match = NULL;
-  GtkTextIter begin, end;
+  GtkTextIter search_pos;
+  GtkTextSearchFlags sflags;
 
-  g_free (pattern);
-  pattern = g_strdup (gtk_entry_get_text (GTK_ENTRY (e)));
-  gtk_widget_destroy (w);
-  gtk_widget_queue_draw (text_view);
+  if (search_bar->case_sensitive)
+    sflags = GTK_TEXT_SEARCH_TEXT_ONLY;
+  else
+    sflags = GTK_TEXT_SEARCH_TEXT_ONLY | GTK_TEXT_SEARCH_CASE_INSENSITIVE;
 
-  if (new_search || gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (text_buffer)))
+  if (search_bar->new_search)
     {
-      /* get the text */
-      g_free (text);
-      gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (text_buffer), &begin, &end);
-      text = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (text_buffer), &begin, &end, FALSE);
-      offset = 0;
-      /* compile new regex */
-      if (regex)
-        g_regex_unref (regex);
-      regex = g_regex_new (pattern, G_REGEX_EXTENDED | G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
-      new_search = FALSE;
-    }
-
-  /* search and select if found */
-  if (g_regex_match (regex, text + offset, G_REGEX_MATCH_NOTEMPTY, &match))
-    {
-      gint sp, ep, spos, epos;
-
-      g_match_info_fetch_pos (match, 0, &sp, &ep);
-
-      /* positions are in bytes, not character, so here we must normalize it */
-      spos = g_utf8_pointer_to_offset (text, text + sp + offset);
-      epos = g_utf8_pointer_to_offset (text, text + ep + offset);
-
-      gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (text_buffer), &begin, spos);
-      gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (text_buffer), &end, epos);
-
-      gtk_text_buffer_select_range (GTK_TEXT_BUFFER (text_buffer), &begin, &end);
-      gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (text_view), &begin, 0, FALSE, 0, 0);
-
-      offset += epos;
-
-      g_match_info_free (match);
-      match = NULL;
+      gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (text_buffer), &search_pos,
+                                        gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (text_buffer)));
     }
   else
-    new_search = TRUE;
-}
+    search_pos = match_end;
 
-static gboolean
-search_key_cb (GtkWidget * w, GdkEventKey * key, GtkWidget * win)
-{
-  if (key->keyval == GDK_KEY_Escape)
+  if (gtk_text_iter_forward_search (&search_pos, search_bar->str, sflags, &match_start, &match_end, NULL))
     {
-      gtk_widget_destroy (win);
-      return TRUE;
+      gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (text_view), &match_start, 0.0, FALSE, 0.0, 0.0);
+      gtk_text_buffer_select_range (GTK_TEXT_BUFFER (text_buffer), &match_start, &match_end);
+      search_changed = TRUE;
     }
-  return FALSE;
+
+  search_bar->new_search = FALSE;
 }
 
 static void
-search_changed (GtkWidget * w, gpointer d)
+do_find_prev (GtkWidget *w, gpointer d)
 {
-  new_search = TRUE;
+  GtkTextIter search_pos;
+  GtkTextSearchFlags sflags;
+
+  if (search_bar->case_sensitive)
+    sflags = GTK_TEXT_SEARCH_TEXT_ONLY;
+  else
+    sflags = GTK_TEXT_SEARCH_TEXT_ONLY | GTK_TEXT_SEARCH_CASE_INSENSITIVE;
+
+  search_pos = match_start;
+  if (gtk_text_iter_backward_search (&search_pos, search_bar->str, sflags, &match_start, &match_end, NULL))
+    {
+      gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (text_view), &match_start, 0.0, FALSE, 0.0, 0.0);
+      gtk_text_buffer_select_range (GTK_TEXT_BUFFER (text_buffer), &match_start, &match_end);
+      search_changed = TRUE;
+    }
 }
 
 static void
-show_search ()
+search_changed_cb (GtkWidget *w, gpointer d)
 {
-  GtkWidget *w, *f, *e;
-  GdkEvent *fev;
+  search_bar->new_search = TRUE;
+  search_bar->str = gtk_entry_get_text (GTK_ENTRY (search_bar->entry));
+  do_find_next (NULL, NULL);
+}
 
-  w = gtk_window_new (GTK_WINDOW_POPUP);
-  gtk_window_set_transient_for (GTK_WINDOW (w), GTK_WINDOW (gtk_widget_get_toplevel (text_view)));
-  gtk_window_set_position (GTK_WINDOW (w), GTK_WIN_POS_CENTER_ON_PARENT);
-  /* next two lines needs for get focus to search window */
-  gtk_window_set_type_hint (GTK_WINDOW (w), GDK_WINDOW_TYPE_HINT_UTILITY);
-  gtk_window_set_modal (GTK_WINDOW (w), TRUE);
+static void
+stop_search_cb (GtkWidget *w, YadSearchBar *sb)
+{
+  ignore_esc = FALSE;
+  gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (search_bar->bar), FALSE);
+  gtk_widget_grab_focus (text_view);
+  if (search_changed)
+    {
+      gtk_text_iter_backward_char (&match_start);
+      gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (text_buffer), &match_start);
+    }
+  search_changed = FALSE;
+}
 
-  g_signal_connect (G_OBJECT (w), "key-press-event", G_CALLBACK (search_key_cb), w);
+/* early prototype for use in open_file_cb() */
+static void fill_buffer_from_file ();
 
-  f = gtk_frame_new (NULL);
-  gtk_frame_set_shadow_type (GTK_FRAME (f), GTK_SHADOW_ETCHED_IN);
-  gtk_container_set_border_width (GTK_CONTAINER (f), 1);
-  gtk_container_add (GTK_CONTAINER (w), f);
+/* file operations */
+static void
+open_file_cb (GtkWidget *w, gpointer d)
+{
+  GtkWidget *dlg;
+  static gchar *dir = NULL;
 
-  e = gtk_entry_new ();
-  if (pattern)
-    gtk_entry_set_text (GTK_ENTRY (e), pattern);
-  gtk_container_add (GTK_CONTAINER (f), e);
+  if (!dir && options.common_data.uri)
+    dir = g_path_get_dirname (options.common_data.uri);
 
-  g_signal_connect (G_OBJECT (e), "activate", G_CALLBACK (do_search), w);
-  g_signal_connect (G_OBJECT (e), "changed", G_CALLBACK (search_changed), NULL);
-  g_signal_connect (G_OBJECT (e), "key-press-event", G_CALLBACK (search_key_cb), w);
+  dlg = gtk_file_chooser_dialog_new (_("YAD - Select File"),
+                                     GTK_WINDOW (gtk_widget_get_toplevel (text_view)),
+                                     GTK_FILE_CHOOSER_ACTION_OPEN,
+                                     _("Cancel"), GTK_RESPONSE_CANCEL,
+                                     _("OK"), GTK_RESPONSE_ACCEPT,
+                                     NULL);
+  if (dir)
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), dir);
 
-  gtk_widget_show_all (w);
+  if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_ACCEPT)
+    {
+      /* set new filename */
+      if (options.common_data.uri)
+        g_free (options.common_data.uri);
+      options.common_data.uri = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg));
 
-  /* send focus event to search entry (so complex due to popup window) */
-  fev = gdk_event_new (GDK_FOCUS_CHANGE);
-  fev->focus_change.type = GDK_FOCUS_CHANGE;
-  fev->focus_change.in = TRUE;
-  fev->focus_change.window = gtk_widget_get_window (e);
-  if (fev->focus_change.window != NULL)
-    g_object_ref (fev->focus_change.window);
-  gtk_widget_send_focus_change (e, fev);
-  gdk_event_free (fev);
+      /* load file */
+      gtk_text_buffer_set_text (GTK_TEXT_BUFFER (text_buffer), "", -1);
+      fill_buffer_from_file ();
+
+      /* keep current dir */
+      g_free (dir);
+      dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dlg));
+
+      text_changed = FALSE;
+    }
+
+  gtk_widget_destroy (dlg);
+}
+
+static void
+save_file_cb (GtkWidget *w, gpointer d)
+{
+  GtkTextIter start, end;
+  gchar *text;
+  GStatBuf st;
+  gint mode = -1;
+  GError *err = NULL;
+
+  gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (text_buffer), &start, &end);
+  text = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (text_buffer), &start, &end, 0);
+
+  /* g_file_set_contents changes the file permissions. so it must be kept and restore after file saving */
+  if (g_stat (options.common_data.uri, &st) == 0)
+    mode = st.st_mode;
+
+  if (!g_file_set_contents (options.common_data.uri, text, -1, &err))
+    {
+      g_printerr ("Cannot save file %s: %s\n", options.common_data.uri, err->message);
+      g_error_free (err);
+    }
+  else
+    {
+      /* restore permissions */
+      if (mode != -1)
+        g_chmod (options.common_data.uri, st.st_mode);
+      text_changed = FALSE;
+    }
+
+  g_free (text);
+}
+
+static void
+save_as_file_cb (GtkWidget *w, gpointer d)
+{
+  GtkWidget *dlg;
+  static gchar *dir = NULL;
+
+  if (!dir && options.common_data.uri)
+    dir = g_path_get_dirname (options.common_data.uri);
+
+  dlg = gtk_file_chooser_dialog_new (_("YAD - Select File"),
+                                     GTK_WINDOW (gtk_widget_get_toplevel (text_view)),
+                                     GTK_FILE_CHOOSER_ACTION_SAVE,
+                                     _("Cancel"), GTK_RESPONSE_CANCEL,
+                                     _("OK"), GTK_RESPONSE_ACCEPT,
+                                     NULL);
+
+  gtk_file_chooser_set_create_folders (GTK_FILE_CHOOSER (dlg), TRUE);
+
+  if (dir)
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), dir);
+
+  if (options.common_data.uri)
+    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dlg), options.common_data.uri);
+  else
+    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dlg), "Untitled.txt");
+
+  if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_ACCEPT)
+    {
+      /* set new filename and save it */
+      if (options.common_data.uri)
+        g_free (options.common_data.uri);
+      options.common_data.uri = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg));
+      save_file_cb (w, d);
+
+      /* keep current dir */
+      g_free (dir);
+      dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dlg));
+    }
+
+  gtk_widget_destroy (dlg);
+}
+
+static void
+quit_cb (GtkWidget *w, gpointer d)
+{
+  yad_exit (options.data.def_resp);
+}
+
+static void
+menu_popup_cb (GtkTextView *w, GtkWidget *popup, gpointer d)
+{
+  if (!GTK_IS_MENU (popup))
+    return;
+
+  if (options.text_data.file_op)
+    {
+      GtkWidget *mitem;
+
+      /* on top */
+      mitem = gtk_separator_menu_item_new ();
+      gtk_menu_shell_prepend (GTK_MENU_SHELL (popup), mitem);
+      gtk_widget_show (mitem);
+
+      if (options.common_data.editable)
+        {
+          mitem = gtk_menu_item_new_with_mnemonic (_("Save _as..."));
+          gtk_menu_shell_prepend (GTK_MENU_SHELL (popup), mitem);
+          g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (save_as_file_cb), NULL);
+          gtk_widget_show (mitem);
+
+          mitem = gtk_menu_item_new_with_mnemonic (_("_Save"));
+          gtk_menu_shell_prepend (GTK_MENU_SHELL (popup), mitem);
+          g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (save_file_cb), NULL);
+          gtk_widget_show (mitem);
+
+          if (!options.common_data.uri)
+            gtk_widget_set_sensitive (mitem, FALSE);
+        }
+
+      mitem = gtk_menu_item_new_with_mnemonic (_("_Open..."));
+      gtk_menu_shell_prepend (GTK_MENU_SHELL (popup), mitem);
+      g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (open_file_cb), NULL);
+      gtk_widget_show (mitem);
+
+      /* at bootom */
+      mitem = gtk_separator_menu_item_new ();
+      gtk_menu_shell_append (GTK_MENU_SHELL (popup), mitem);
+      gtk_widget_show (mitem);
+
+      mitem = gtk_menu_item_new_with_mnemonic (_("_Quit"));
+      gtk_menu_shell_append (GTK_MENU_SHELL (popup), mitem);
+      g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (quit_cb), NULL);
+      gtk_widget_show (mitem);
+    }
 }
 
 static gboolean
-key_press_cb (GtkWidget * w, GdkEventKey * key, gpointer data)
+key_press_cb (GtkWidget *w, GdkEventKey *key, gpointer d)
 {
-  if ((key->state & GDK_CONTROL_MASK) && (key->keyval == GDK_KEY_S || key->keyval == GDK_KEY_s))
+  if ((key->state & GDK_CONTROL_MASK) && (key->keyval == GDK_KEY_F || key->keyval == GDK_KEY_f))
     {
-      show_search ();
-      return TRUE;
+      if (search_bar == NULL)
+        return FALSE;
+
+      ignore_esc = TRUE;
+      gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (search_bar->bar), TRUE);
+      return gtk_search_bar_handle_event (GTK_SEARCH_BAR (search_bar->bar), (GdkEvent *) key);
+    }
+  else if (options.text_data.file_op)
+    {
+      if ((key->state & GDK_CONTROL_MASK) && (key->keyval == GDK_KEY_O || key->keyval == GDK_KEY_o))
+        {
+          open_file_cb (NULL, NULL);
+          return TRUE;
+        }
+      else if ((key->state & GDK_CONTROL_MASK) && (key->keyval == GDK_KEY_S || key->keyval == GDK_KEY_s))
+        {
+          save_file_cb (NULL, NULL);
+          return TRUE;
+        }
+      else if ((key->state & GDK_CONTROL_MASK) && (key->state & GDK_SHIFT_MASK) &&
+               (key->keyval == GDK_KEY_S || key->keyval == GDK_KEY_s))
+        {
+          save_as_file_cb (NULL, NULL);
+          return TRUE;
+        }
+      else if ((key->state & GDK_CONTROL_MASK) && (key->keyval == GDK_KEY_Q || key->keyval == GDK_KEY_q))
+        {
+          quit_cb (NULL, NULL);
+          return TRUE;
+        }
     }
 
   return FALSE;
 }
 
+/* mouse actions */
 static gboolean
 tag_event_cb (GtkTextTag * tag, GObject * obj, GdkEvent * ev, GtkTextIter * iter, gpointer d)
 {
@@ -260,6 +420,34 @@ linkify_cb (GtkTextBuffer * buf, GRegex * regex)
   g_free (text);
 }
 
+#if HAVE_SOURCEVIEW
+static void
+line_mark_activated (GtkSourceGutter *gutter, GtkTextIter *iter, GdkEventButton *ev, gpointer d)
+{
+  GSList *mark_list;
+  const gchar *mark_type;
+
+  if (ev->button == 1)
+    mark_type = SV_MARK1;
+  else if (ev->button == 3)
+    mark_type = SV_MARK2;
+  else
+    return;
+
+  /* get the marks already in the line */
+  mark_list = gtk_source_buffer_get_source_marks_at_line (GTK_SOURCE_BUFFER (text_buffer),
+                                                          gtk_text_iter_get_line (iter), mark_type);
+
+  if (mark_list != NULL)
+    gtk_text_buffer_delete_mark (GTK_TEXT_BUFFER (text_buffer), GTK_TEXT_MARK (mark_list->data));
+  else
+    gtk_source_buffer_create_source_mark (GTK_SOURCE_BUFFER (text_buffer), NULL, mark_type, iter);
+
+  g_slist_free (mark_list);
+}
+#endif
+
+/* data loading */
 static gboolean
 handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
 {
@@ -402,6 +590,13 @@ fill_buffer_from_file ()
     lang = gtk_source_language_manager_guess_language (gtk_source_language_manager_get_default (), options.common_data.uri, NULL);
   gtk_source_buffer_set_language (GTK_SOURCE_BUFFER (text_buffer), lang);
 #endif
+
+  if (options.common_data.editable)
+    {
+      /* move cursor at the beginnging of a buffer */
+      gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (text_buffer), &iter, 0);
+      gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (text_buffer), &iter);
+    }
 }
 
 static void
@@ -415,14 +610,23 @@ fill_buffer_from_stdin ()
   g_io_add_watch (channel, G_IO_IN | G_IO_HUP, handle_stdin, NULL);
 }
 
+static void
+text_changed_cb (GtkTextBuffer *b, gpointer d)
+{
+  text_changed = TRUE;
+}
+
 GtkWidget *
 text_create_widget (GtkWidget * dlg)
 {
-  GtkWidget *w, *tv;
+  GtkWidget *w, *sw, *tv;
 
-  w = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (w), GTK_SHADOW_ETCHED_IN);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (w), options.hscroll_policy, options.vscroll_policy);
+  w = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+
+  sw = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_ETCHED_IN);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), options.hscroll_policy, options.vscroll_policy);
+  gtk_box_pack_start (GTK_BOX (w), sw, TRUE, TRUE, 0);
 
 #ifdef HAVE_SOURCEVIEW
   text_buffer = (GObject *) gtk_source_buffer_new (NULL);
@@ -442,6 +646,8 @@ text_create_widget (GtkWidget * dlg)
   if (options.text_data.wrap)
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD_CHAR);
 
+  gtk_text_view_set_monospace (GTK_TEXT_VIEW (text_view), TRUE);
+
   if (options.common_data.font || options.text_data.fore || options.text_data.back)
     {
       GtkCssProvider *provider;
@@ -470,7 +676,7 @@ text_create_widget (GtkWidget * dlg)
     }
 
 #ifdef HAVE_SOURCEVIEW
-  if (options.source_data.theme)
+  if (options.source_data.theme && *options.source_data.theme)
     {
       GtkSourceStyleScheme *scheme = NULL;
       GtkSourceStyleSchemeManager *mgr;
@@ -505,6 +711,49 @@ text_create_widget (GtkWidget * dlg)
       else
         g_printerr (_("Theme %s not found\n"), options.source_data.theme);
     }
+
+  gtk_source_view_set_show_line_numbers (GTK_SOURCE_VIEW (text_view), options.source_data.line_num);
+  gtk_source_view_set_highlight_current_line (GTK_SOURCE_VIEW (text_view), options.source_data.line_hl);
+  if (options.source_data.line_marks)
+    {
+      GdkRGBA color;
+      GtkSourceMarkAttributes *attr;
+
+      gtk_source_view_set_show_line_marks (GTK_SOURCE_VIEW (text_view), TRUE);
+
+      gdk_rgba_parse (&color, options.source_data.m1_color);
+      attr = gtk_source_mark_attributes_new ();
+      gtk_source_mark_attributes_set_background (attr, &color);
+      gtk_source_mark_attributes_set_icon_name (attr, "checkbox-checked-symbolic");
+      gtk_source_view_set_mark_attributes (GTK_SOURCE_VIEW (text_view), SV_MARK1, attr, 0);
+      g_object_unref (attr);
+
+      gdk_rgba_parse (&color, options.source_data.m2_color);
+      attr = gtk_source_mark_attributes_new ();
+      gtk_source_mark_attributes_set_background (attr, &color);
+      gtk_source_mark_attributes_set_icon_name (attr, "checkbox-checked-symbolic");
+      gtk_source_view_set_mark_attributes (GTK_SOURCE_VIEW (text_view), SV_MARK2, attr, 0);
+      g_object_unref (attr);
+
+      g_signal_connect (G_OBJECT (text_view), "line-mark-activated", G_CALLBACK (line_mark_activated), NULL);
+    }
+  if (options.source_data.right_margin > 0)
+    {
+      gtk_source_view_set_show_right_margin (GTK_SOURCE_VIEW (text_view), TRUE);
+      gtk_source_view_set_right_margin_position (GTK_SOURCE_VIEW (text_view), options.source_data.right_margin);
+    }
+  if (options.source_data.indent)
+    {
+      gtk_source_view_set_auto_indent (GTK_SOURCE_VIEW (text_view), TRUE);
+      gtk_source_view_set_indent_on_tab (GTK_SOURCE_VIEW (text_view), TRUE);
+    }
+  gtk_source_view_set_tab_width (GTK_SOURCE_VIEW (text_view), options.source_data.tab_width);
+  gtk_source_view_set_indent_width (GTK_SOURCE_VIEW (text_view), options.source_data.indent_width);
+  gtk_source_view_set_smart_home_end (GTK_SOURCE_VIEW (text_view), options.source_data.smart_he);
+  gtk_source_view_set_smart_backspace (GTK_SOURCE_VIEW (text_view), options.source_data.smart_bs);
+  gtk_source_view_set_insert_spaces_instead_of_tabs (GTK_SOURCE_VIEW (text_view), options.source_data.spaces);
+
+  gtk_source_buffer_set_highlight_matching_brackets (GTK_SOURCE_BUFFER (text_buffer), options.source_data.brackets);
 #endif
 
 #ifdef HAVE_SPELL
@@ -515,9 +764,15 @@ text_create_widget (GtkWidget * dlg)
     }
 #endif
 
-  /* Add submit on ctrl+enter */
+  /* Add keyboard handler */
   g_signal_connect (text_view, "key-press-event", G_CALLBACK (key_press_cb), dlg);
 
+  if (options.text_data.file_op)
+    {
+      g_signal_connect (G_OBJECT (text_buffer), "changed", G_CALLBACK (text_changed_cb), NULL);
+      g_signal_connect_after (G_OBJECT (text_view), "populate-popup", G_CALLBACK (menu_popup_cb), dlg);
+    }
+
   /* Initialize linkifying */
   if (options.text_data.uri)
     {
@@ -542,8 +797,22 @@ text_create_widget (GtkWidget * dlg)
       g_signal_connect_after (G_OBJECT (text_buffer), "changed", G_CALLBACK (linkify_cb), regex);
     }
 
-  gtk_container_add (GTK_CONTAINER (w), tv);
+  gtk_container_add (GTK_CONTAINER (sw), tv);
 
+  /* create search bar */
+  if (options.common_data.enable_search)
+    {
+      if ((search_bar = create_search_bar ()) != NULL)
+        {
+          gtk_box_pack_start (GTK_BOX (w), search_bar->bar, FALSE, FALSE, 0);
+          g_signal_connect (G_OBJECT (search_bar->entry), "search-changed", G_CALLBACK (search_changed_cb), NULL);
+          g_signal_connect (G_OBJECT (search_bar->entry), "stop-search", G_CALLBACK (stop_search_cb), NULL);
+          g_signal_connect (G_OBJECT (search_bar->entry), "next-match", G_CALLBACK (do_find_next), NULL);
+          g_signal_connect (G_OBJECT (search_bar->entry), "previous-match", G_CALLBACK (do_find_prev), NULL);
+        }
+    }
+
+  /* load data */
   if (options.common_data.uri)
     fill_buffer_from_file ();
 
@@ -564,14 +833,31 @@ text_create_widget (GtkWidget * dlg)
 void
 text_print_result (void)
 {
-  GtkTextIter start, end;
-  gchar *text;
-
   if (!options.common_data.editable)
     return;
 
-  gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (text_buffer), &start, &end);
-  text = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (text_buffer), &start, &end, 0);
-  g_print ("%s", text);
-  g_free (text);
+  if (options.text_data.in_place && options.common_data.uri)
+    {
+      if (text_changed)
+        {
+          if (options.text_data.confirm_save)
+            {
+              if (yad_confirm_dlg (GTK_WINDOW (gtk_widget_get_toplevel (text_view)),
+                                   options.text_data.confirm_text))
+                save_file_cb (NULL, NULL);
+            }
+          else
+            save_file_cb (NULL, NULL);
+        }
+    }
+  else
+    {
+      GtkTextIter start, end;
+      gchar *text;
+
+      gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (text_buffer), &start, &end);
+      text = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (text_buffer), &start, &end, 0);
+      g_print ("%s", text);
+      g_free (text);
+    }
 }
