@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2021, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #ifndef _GNU_SOURCE
@@ -239,7 +239,7 @@ update_preview (GtkFileChooser * chooser, GtkWidget *p)
 }
 
 gchar **
-split_arg (const gchar * str)
+split_arg (const gchar *str)
 {
   gchar **res;
   gchar *p_col;
@@ -637,31 +637,56 @@ print_bool_val (gboolean val)
   return ret;
 }
 
-gint
-run_command_sync (gchar *cmd, gchar **out)
+typedef struct {
+  gchar *cmd;
+  gchar **out;
+} RunData;
+
+static gboolean run_lock = FALSE;
+static gint ret = 0;
+
+static void
+run_thread (RunData *d)
 {
-  gint ret = 0;
-  gchar *full_cmd = NULL;
   GError *err = NULL;
 
-  if (options.data.use_interp)
-    {
-      if (g_strstr_len (options.data.interp, -1, "%s") != NULL)
-        full_cmd = g_strdup_printf (options.data.interp, cmd);
-      else
-        full_cmd = g_strdup_printf ("%s %s", options.data.interp, cmd);
-    }
-  else
-    full_cmd = g_strdup (cmd);
-
-  if (!g_spawn_command_line_sync (full_cmd, out, NULL, &ret, &err))
+  if (!g_spawn_command_line_sync (d->cmd, d->out, NULL, &ret, &err))
     {
       if (options.debug)
         g_printerr (_("WARNING: Run command failed: %s\n"), err->message);
       g_error_free (err);
     }
+  run_lock = FALSE;
+}
+
+gint
+run_command_sync (gchar *cmd, gchar **out)
+{
+  RunData *d;
+
+  d = g_new0 (RunData, 1);
+
+  if (options.data.use_interp)
+    {
+      if (g_strstr_len (options.data.interp, -1, "%s") != NULL)
+        d->cmd = g_strdup_printf (options.data.interp, cmd);
+      else
+        d->cmd = g_strdup_printf ("%s %s", options.data.interp, cmd);
+    }
+  else
+    d->cmd = g_strdup (cmd);
+  d->out = out;
+
+  run_lock = TRUE;
+  ret = 0;
+  g_thread_new ("run_sync", (GThreadFunc) run_thread, d);
+
+  while (run_lock != FALSE)
+    gtk_main_iteration ();
+
+  g_free (d->cmd);
+  g_free (d);
 
-  g_free (full_cmd);
   return ret;
 }
 
@@ -799,37 +824,92 @@ open_uri (const gchar *uri)
   g_free (cmdline);
 }
 
-#ifdef HAVE_SPELL
-void
-show_langs ()
+/* Search bar */
+static void
+next_clicked_cb (GtkWidget *w, YadSearchBar *sb)
 {
-  const GList *lng;
-
-  for (lng = gspell_language_get_available (); lng; lng = lng->next)
-    {
-      const GspellLanguage *l = lng->data;
-      g_print ("%s\n", gspell_language_get_code (l));
-    }
+  g_signal_emit_by_name (sb->entry, "next-match");
 }
-#endif
 
-#ifdef HAVE_SOURCEVIEW
-void
-show_themes ()
+static void
+prev_clicked_cb (GtkWidget *w, YadSearchBar *sb)
 {
-  GtkSourceStyleSchemeManager *sm;
-  const gchar **si;
-  guint i = 0;
-
-  sm = gtk_source_style_scheme_manager_get_default ();
-  if ((si = (const gchar **) gtk_source_style_scheme_manager_get_scheme_ids (sm)) == NULL)
-    return;
-
-  while (si[i])
-    {
-      GtkSourceStyleScheme *s = gtk_source_style_scheme_manager_get_scheme (sm, si[i]);
-      g_print ("%s\n", gtk_source_style_scheme_get_name (s));
-      i++;
-    }
+  g_signal_emit_by_name (sb->entry, "previous-match");
 }
+
+static void
+case_toggle_cb (GtkToggleButton *b, YadSearchBar *sb)
+{
+  sb->case_sensitive = !sb->case_sensitive;
+  gtk_toggle_button_set_active (b, sb->case_sensitive);
+}
+
+YadSearchBar *
+create_search_bar ()
+{
+  YadSearchBar *sb;
+  GtkWidget *b;
+  gint e_width = -1;
+
+  sb = g_new0 (YadSearchBar, 1);
+  sb->new_search = TRUE;
+
+  if (!sb)
+    return NULL;
+
+  sb->bar = gtk_search_bar_new ();
+  gtk_search_bar_set_show_close_button (GTK_SEARCH_BAR (sb->bar), TRUE);
+
+  b = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+  gtk_container_add (GTK_CONTAINER (sb->bar), b);
+
+  sb->entry = gtk_search_entry_new ();
+  gtk_box_pack_start (GTK_BOX (b), sb->entry, TRUE, TRUE, 0);
+  gtk_search_bar_connect_entry (GTK_SEARCH_BAR (sb->bar), GTK_ENTRY (sb->entry));
+
+#ifndef STANDALONE
+  e_width = g_settings_get_int (settings, "search-width");
 #endif
+  if (e_width > 0)
+    gtk_widget_set_size_request (sb->entry, e_width, -1);
+
+  sb->next = gtk_button_new_from_icon_name ("go-down", GTK_ICON_SIZE_BUTTON);
+  gtk_widget_set_focus_on_click (sb->next, FALSE);
+  gtk_box_pack_start (GTK_BOX (b), sb->next, FALSE, FALSE, 0);
+
+  g_signal_connect (G_OBJECT (sb->next), "clicked", G_CALLBACK (next_clicked_cb), sb);
+
+  sb->prev = gtk_button_new_from_icon_name ("go-up", GTK_ICON_SIZE_BUTTON);
+  gtk_widget_set_focus_on_click (sb->prev, FALSE);
+  gtk_box_pack_start (GTK_BOX (b), sb->prev, FALSE, FALSE, 0);
+
+  g_signal_connect (G_OBJECT (sb->prev), "clicked", G_CALLBACK (prev_clicked_cb), sb);
+
+  sb->case_toggle = gtk_check_button_new_with_mnemonic (_("Case _sensitive"));
+  gtk_widget_set_focus_on_click (sb->case_toggle, FALSE);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sb->case_toggle), sb->case_sensitive);
+  gtk_box_pack_start (GTK_BOX (b), sb->case_toggle, FALSE, FALSE, 0);
+
+  g_signal_connect (G_OBJECT (sb->case_toggle), "toggled", G_CALLBACK (case_toggle_cb), sb);
+
+  return sb;
+}
+
+/* Confirmation dialog */
+gboolean
+yad_confirm_dlg (GtkWindow *parent, gchar *txt)
+{
+  GtkWidget *d;
+  gchar *buf;
+
+  buf = g_strcompress (options.text_data.confirm_text);
+  d = gtk_message_dialog_new (parent, GTK_DIALOG_DESTROY_WITH_PARENT,
+                              GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, "%s", buf);
+  gtk_window_set_position (GTK_WINDOW (d), GTK_WIN_POS_CENTER_ON_PARENT);
+  g_free (buf);
+
+  ret = gtk_dialog_run (GTK_DIALOG (d));
+  gtk_widget_destroy (d);
+
+  return (ret == GTK_RESPONSE_YES);
+}
