@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2021, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #ifndef _YAD_H_
@@ -23,7 +23,9 @@
 #include <config.h>
 
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/ipc.h>
+#include <fcntl.h>
 
 #include <gdk/gdkx.h>
 
@@ -33,6 +35,7 @@
 
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
+#include <glib/gstdio.h>
 
 #ifdef HAVE_HTML
 #include <webkit2/webkit2.h>
@@ -59,6 +62,11 @@ G_BEGIN_DECLS
 
 #define YAD_URL_REGEX "(http|https|ftp|file)://[a-zA-Z0-9./_%#&-]+"
 
+#define RIGHT_MARGIN     80  /* default right margin position for GtkSourceView */
+
+#define SV_MARK1 "one"
+#define SV_MARK2 "two"
+
 typedef enum {
   YAD_MODE_MESSAGE,
   YAD_MODE_APP,
@@ -112,6 +120,7 @@ typedef enum {
   YAD_FIELD_MDIR,
   YAD_FIELD_FONT,
   YAD_FIELD_APP,
+  YAD_FIELD_ICON,
   YAD_FIELD_COLOR,
   YAD_FIELD_DATE,
   YAD_FIELD_SCALE,
@@ -240,6 +249,7 @@ typedef struct {
   gboolean use_interp;
   gchar *interp;
   gchar *uri_handler;
+  gchar *f1_action;
   /* window settings */
   gboolean sticky;
   gboolean fixed;
@@ -262,6 +272,17 @@ typedef struct {
   gboolean extended;
 } YadAppData;
 
+typedef struct {
+  gchar *name;
+  gchar *version;
+  gchar *copyright;
+  gchar *comments;
+  gchar *license;
+  gchar *authors;
+  gchar *website;
+  gchar *website_lbl;
+} YadAboutData;
+
 typedef struct {
   gint day;
   gint month;
@@ -318,6 +339,9 @@ typedef struct {
   gboolean output_by_row;
   guint focus_field;
   gboolean cycle_read;
+  gboolean align_buttons;
+  gchar *changed_action;
+  gboolean homogeneous;
 } YadFormData;
 
 #ifdef HAVE_HTML
@@ -329,6 +353,7 @@ typedef struct {
   gchar *encoding;
   gchar *user_agent;
   gchar *user_style;
+  gchar **wk_props;
 } YadHtmlData;
 #endif
 
@@ -378,11 +403,12 @@ typedef struct {
 } YadListData;
 
 typedef struct {
-  GSList *tabs;
+  gchar **tabs;
   guint borders;
   GtkPositionType pos;
   guint active;
   gboolean expand;
+  gboolean stack;
 } YadNotebookData;
 
 #ifdef HAVE_TRAY
@@ -396,6 +422,7 @@ typedef struct {
 typedef struct {
   GtkOrientation orient;
   gint splitter;
+  gint focused;
 } YadPanedData;
 
 typedef struct {
@@ -449,12 +476,29 @@ typedef struct {
   gboolean formatted;
   gchar *fore;
   gchar *back;
+  gboolean in_place;
+  gboolean file_op;
+  gboolean confirm_save;
+  gchar *confirm_text;
 } YadTextData;
 
 #ifdef HAVE_SOURCEVIEW
 typedef struct {
   gchar *lang;
   gchar *theme;
+  gboolean line_num;
+  gboolean line_hl;
+  gboolean line_marks;
+  gchar *m1_color;
+  gchar *m2_color;
+  guint right_margin;
+  gboolean brackets;
+  gboolean indent;
+  gint tab_width;
+  gint indent_width;
+  GtkSourceSmartHomeEndType smart_he;
+  gboolean smart_bs;
+  gboolean spaces;
 } YadSourceData;
 #endif
 
@@ -479,6 +523,7 @@ typedef struct {
   gboolean num_output;
   gboolean hide_text;
   gint icon_size;
+  gboolean enable_search;
 #if GLIB_CHECK_VERSION(2,30,0)
   GFormatSizeFlags size_fmt;
 #endif
@@ -498,6 +543,7 @@ typedef struct {
   YadData data;
   YadCommonData common_data;
 
+  YadAboutData about_data;
   YadAppData app_data;
   YadCalendarData calendar_data;
   YadColorData color_data;
@@ -546,16 +592,31 @@ typedef struct {
 #endif
 } YadOptions;
 
+/* Searvh bar */
+typedef struct {
+  GtkWidget *bar;
+  GtkWidget *entry;
+  GtkWidget *next;
+  GtkWidget *prev;
+  GtkWidget *case_toggle;
+  gboolean case_sensitive;
+  gboolean new_search;
+  const gchar *str;
+} YadSearchBar;
+
 extern YadOptions options;
 extern GtkIconTheme *yad_icon_theme;
 
 #ifndef STANDALONE
 extern GSettings *settings;
+extern GSettings *sv_settings;
 #endif
 
 extern GdkPixbuf *big_fallback_image;
 extern GdkPixbuf *small_fallback_image;
 
+extern gboolean ignore_esc;
+
 /* TABS */
 typedef struct {
   pid_t pid;
@@ -665,17 +726,13 @@ gchar *print_bool_val (gboolean val);
 gint run_command_sync (gchar *cmd, gchar **out);
 void run_command_async (gchar *cmd);
 
-gchar * pango_to_css (gchar *font);
+gchar *pango_to_css (gchar *font);
 
 void open_uri (const gchar *uri);
 
-#ifdef HAVE_SPELL
-void show_langs ();
-#endif
+YadSearchBar *create_search_bar ();
 
-#ifdef HAVE_SOURCEVIEW
-void show_themes ();
-#endif
+gboolean yad_confirm_dlg (GtkWindow *parent, gchar *txt);
 
 static inline void
 strip_new_line (gchar * str)
